// cam.cpp -- handles cam server

#ifdef WIN32
#include <process.h>
#include "win32.h"
#else
#include <unistd.h>
#include <pthread.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/poll.h>
#endif

#include <fcntl.h>

#include "cam.h"
#include "common.h"
#include "box.h"
#include "btkcam-client.h"

#define MIN_ECM_PROCESSING_TIME 150000 //microseconds
#define sleep( milliseconds ) usleep( (milliseconds) * 1000 )

extern 		statistics *stats; 	
extern 		btkcam_client *btk[10];
extern char	Blines[10][255];
extern int 	Bcnt;
extern int	operation_mode;
extern int	operation_prio;
extern int	config_log_to_console;

extern bisskeys 	bisslist[100];				// BISS & DCW list in dcwbiss.cfg
extern int		bisscnt;


char caid_descriptions[0x49][0x30] = {
	"Free to Air", "Seca", "CCETT", "Deutsche Telecom", "Eurodec", "Viaccess", "Irdeto", 
	"Jerrold/GI", "Matra", "NDS", "Nokia", "Conax", "NTL", "CryptoWorks", "PowerVu", "Sony",
	"Tandberg", "Thompson", "TV/Com (NagraVision Bell ExpressVU)", "HPT", "HRT", "IBM", 
	"Nera", "BetaCrypt", "NagraVision", "Titan", "Unknown", "Unknown", "Unknown", "Unknown",
	"Unknown", "Unknown", "Telefonica", "Stentor", "Tadiran Scopus", "BARCO AS", "StarGuide",
	"Mentor",	"EBU", "Unknown", "Unknown", "Unknown", "Unknown", "Unknown", "Unknown", 
	"Unknown", "Unknown", "Unknown", "Unknown", "Unknown", "Unknown", "Unknown", "Unknown",
	"Unknown", "Unknown", "Unknown", "Unknown", "Unknown", "Unknown", "Unknown", "Unknown",
	"Unknown", "Unknown", "Unknown", "Unknown", "Unknown", "Unknown", "Unknown", "Unknown", 
	"Unknown", "Unknown", "GI", "Telemann"
};

//-- initialization of static members of service class -------------
service* service::service_root = NULL;
int service::listener_fd = -1;
service_status_t service::listener_status = DISCONNECTED;
pthread_mutex_t service::cardserver_lock = PTHREAD_MUTEX_INITIALIZER;

//-------------------------------------------------------------------


/* static */ int service::initialize( void )
{
	// ------------------------------ listening socket initialization ---------------------------------
	//_slog(log_trans, _DEBUG_BASIC, 1, "INFO:: Setting up listening cam server... ", 1);
	
	struct sockaddr_un socket_address;
	socket_address.sun_family = AF_UNIX;
	
	strncpy(socket_address.sun_path, box::get_cam_socket_path(), sizeof(socket_address.sun_path) - 1);
	socket_address.sun_path[sizeof(socket_address.sun_path) - 1] = '\0';
	unlink(socket_address.sun_path);
	chmod(socket_address.sun_path, S_IRWXU | S_IRWXG | S_IRWXO);
	
	if((listener_fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
	{
		_slog(NO_TRANS, _DEBUG_BASIC, 0, "error opening socket.", 1);
        	return 1;
	}
	
	if( bind(listener_fd, (struct sockaddr *)&socket_address, sizeof(socket_address.sun_family) + 
    		strlen(socket_address.sun_path) ) < 0 )
	{
		_slog(NO_TRANS, _DEBUG_BASIC, 0, "error binding to socket.", 1);
		close(listener_fd);
		listener_fd = -1;
        	return 2;
	}
	
	if( listen(listener_fd, 5) < 0 )
	{
		_slog(NO_TRANS, _DEBUG_BASIC, 0, "listen error.", 1);
		close(listener_fd);
		listener_fd = -1;
		return 3;
	}
	
	service::listener_status = SLEEPING;
	return 0;
}

/* static */ void service::finalize( void )
{
	stop_all_decoders();
	close(listener_fd);
	unlink(box::get_cam_socket_path());
	box::close_ca_devices();
}

/* static */ int service::count( void )
{
	int service_count = 0;
	service* service_ptr = service_root;
	while( service_ptr )
	{
		service_count++;
		service_ptr = service_ptr->next;
	}
	return service_count;
}

/* static */ void service::remove_inactive( void )
{
	//this removes any service that for some reason it's decoder has
	//stopped other than client disconnection.
	service* service_tmp;
	service* service_ptr = service_root;
	while( service_ptr )
	{
		if( !service_ptr->decoder_running )
		{
			service_tmp = service_ptr->next;
			_slog(NO_TRANS, _DEBUG_BASIC, 1, _logmf("INFO:: Removing inactive service (SID: %04d).", service_ptr->sid), 1);
			service::remove( service_ptr );
			service_ptr = service_tmp;
		}
		else service_ptr = service_ptr->next;
	}
}

/* static */ bool service::accept_service_request( void )
{
	int message_fd;
	struct sockaddr_un client_address;
	socklen_t client_address_length;
	
	if ((message_fd = accept(listener_fd, (struct sockaddr *)&client_address, &client_address_length)) < 0)
	{
		_slog(NO_TRANS, _DEBUG_BASIC, 1, "INFO:: Socket accept error, exiting.", 1);
		return false;
	}
	listener_status = SLEEPING; //now that the connections has been accepted, reset to SLEEPING.
	
	//_slog(NO_TRANS, _DEBUG_BASIC, 1, _logmf("INFO:: Service request received (descriptor: %d)... ", message_fd), 1);
	service* request_handler = new service( message_fd );
	add( request_handler );
	request_handler->process_message();
	
	return true;
}

/* static */ int service::poll_services( void )
{
	service* service_ptr = service_root;
	int i, poll_status;
	int poll_count = service::count() + 1;

	pollfd *poll_fds = new pollfd[poll_count];
	poll_fds[0].events = (POLLIN | POLLPRI);
	poll_fds[0].fd = listener_fd;
	
	i = 1;
	while( service_ptr )
	{
		poll_fds[i].events = (POLLIN | POLLPRI | POLLHUP);
		poll_fds[i].fd = service_ptr->message_fd;
		service_ptr = service_ptr->next;
		i++;
	}
	
	poll_status = poll(poll_fds, poll_count, 250); //poll every 250ms
	if( poll_status > 0 )
	{
		for(i=0;i<poll_count;i++)
		{
			if( fd_is_listener( poll_fds[i].fd ) )
			{
				if( poll_fds[i].revents ) listener_status = DATA_AVAILABLE;
			}
			else
			{
				service_ptr = get_service_by_fd( poll_fds[i].fd );
				if( service_ptr )
				{
					if( poll_fds[i].revents & POLLHUP )
						service_ptr->status = DISCONNECTED;
					else if( poll_fds[i].revents & (POLLIN | POLLPRI) )
						service_ptr->status = DATA_AVAILABLE;
				}
				else _slog(NO_TRANS, _DEBUG_BASIC, 1, "INFO:: No service running for requested file descriptor.", 1);
			}
		}
	}

	delete poll_fds;
	return poll_status;
}

/* static */ service* service::get_service_by_fd( int fd )
{
	service* service_ptr = service_root;
	while( service_ptr )
	{
		if( service_ptr->message_fd == fd ) break;
		service_ptr = service_ptr->next;
	}
	return service_ptr;
}

/* static */ bool service::fd_is_listener( int fd )
{
	return (listener_fd == fd);
}

/* static */ void service::add( service* new_service )
{
	service* service_ptr = service_root;
	
	if( service_ptr )
	{
		while( service_ptr->next ) { service_ptr = service_ptr->next; }
		service_ptr->next = new_service;
		new_service->previous = service_ptr;
	}
	else
	{
		service_root = new_service;
	}
}

/* static */ void service::remove( service* remove_service )
{
	service* service_ptr = service_root;
	service* service_tmp;
	
	if( service_ptr == remove_service )
	{
		service_root = service_ptr->next;
		if( service_root ) service_root->previous = NULL;
		delete service_ptr;
		return;
	}
		
	while( service_ptr )
	{
		if( service_ptr == remove_service )
		{
			service_tmp = service_ptr->next;
			service_ptr->previous->next = service_tmp;
			delete service_ptr;
			break;
		}
		else service_ptr = service_ptr->next;
	}
}



/* static */ void service::stop_all_decoders(void)
{
	service* service_ptr = service_root;
	
	while( service_ptr )
	{
		service_ptr->stop_decoder();
		service_ptr = service_ptr->next;
	}
}

/* static */ int service::get_free_descramble_index(void)
{
	const int MAX_INDEX = 8;
	bool in_use;
	int descramble_index = 0;
	service* service_ptr = service_root;
	
	while( 1 )
	{
		in_use = false;
		while( service_ptr )
		{
			if( descramble_index == service_ptr->descramble_index )
			{
				in_use = true;
				break;
			}
			service_ptr = service_ptr->next;
		}
		
		if( in_use )
		{
			descramble_index++;
			if( descramble_index == MAX_INDEX ) return -1;
		}	
		else break;
	}
	return descramble_index;
}

/* static */ void service::refresh_ca_pids( void )
{
	service* service_ptr = service_root;
	
	while( service_ptr )
	{
		service_ptr->set_ca_pids();
		service_ptr = service_ptr->next;
	}
}


service::service(int accepted_message_fd)
{
	log_trans = 0;

	
	previous = NULL;
	next = NULL;
	
	status = SLEEPING;
	

	message_fd = accepted_message_fd;
	message_length = 0;

	caid = 0;

	provider_id = 0;
	
	//service related
	sid = 0;
	dvb_namespace = 0;
	tsid = 0;
	onid = 0;
	pmt_pid = 0;

	descramble_index = -1;
	demux_index = 0; //use demux0 initially
	descramble_mask = 1; //use ca0 initially
	pid_sections_count = 0;
	
	root_ca_info_count = 0;
	
	// ecm related
	failed_ecm_decryption_count = 0;
	last_ecm_table = 0;	
	ecm_data_length = 0;
	ecm_table = 0;
	ecm_pid = 0;
	ecm_demux_fd = -1;
	ecm_thread = 0;


	decoder_running = 1;


	memset(cw_0_plain, 0, 8);
	memset(cw_1_plain, 0, 8);
	memset(last_cw_0_plain, 0, 8);
	memset(last_cw_1_plain, 0, 8);

	memset(cw_0, 0, 8);
	memset(cw_1, 0, 8);
	memset(last_cw_0, 0, 8);
	memset(last_cw_1, 0, 8);


}

service::~service()
{
	stop_decoder();
	close( message_fd );
}

int service::fd_peek( int fd, unsigned int wait )
{
	struct pollfd poll_fds[1];
	poll_fds[0].fd = fd;
	poll_fds[0].events = (POLLIN | POLLPRI);

	return poll(poll_fds, 1, wait);
}

void *service::process_ecm_thread( void *arg )
{
	double ecm_start_time, ecm_compensation_time;
	unsigned char raw_ecm_data[512];
	int raw_ecm_data_length;
	
	// card-server related
	bool immediate_ecm_request = false;

	
	service* handler = (service*)arg;
	
	// setup cancellation behavior
	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
	box::write_control_words( handler );
	if (handler->Algo==0)
		while (1) sleep(100);
		
	if (handler->Algo==1) while (1)
		{
		sleep(2500);
		pthread_mutex_lock( &service::cardserver_lock );
		btk[handler->ActB]->checknet((unsigned char *)handler->btkcmd,16);
		if (btk[handler->ActB]->status==0) 
			   if (memcmp(btk[handler->ActB]->answer,"KEY:",4) == 0) 
				{
				sscanf(btk[handler->ActB]->answer,"KEY:%s",handler->keyret);	
				keystr_to_key(handler->keyret,handler->cw_0,handler->cw_1);
				box::write_control_words( handler );
				memcpy(handler->last_cw_0,handler->cw_0, 8);
				memcpy(handler->last_cw_1,handler->cw_1, 8);
				_log_commit_trans( handler->log_trans );
				}
		pthread_mutex_unlock( &service::cardserver_lock );
		}
		
	// open demux device.
	if( (handler->ecm_demux_fd = box::get_demux_device( handler->demux_index, handler )) <= 0 )
	{
		handler->decoder_running = 0;
		return NULL;
	}
	
#ifndef __DEMUX_WORKAROUND__
	if( (box::start_filter( handler->ecm_demux_fd, handler->ecm_pid, handler->ecm_table, 0xF0, FILTER_TIMEOUT, handler )) < 0 )
	{
		handler->decoder_running = 0;
		return NULL;
	}
#else
	sleep(10);
#endif
	
	while( 1 )
	{
#ifdef __DEMUX_WORKAROUND__
#warning "Compiling with __DEMUX_WORKAROUND__"
		box::start_filter( handler->ecm_demux_fd, handler->ecm_pid, handler->ecm_table, 0xF0, FILTER_TIMEOUT, handler );
		sleep(10);
#endif
		if( handler->fd_peek( handler->ecm_demux_fd, FILTER_PEEK_TIMEOUT) <= 0 )
		{
			//if time-out occurred, restart the filter and try again.
			_slog(handler->log_trans, _DEBUG_BASIC, 1, _logmf("cam[di:%d]: Demux timeout. Restarting filter.", handler->descramble_index), 1);
#ifdef __DEMUX_WORKAROUND__
#warning "Compiling with __DEMUX_WORKAROUND__"
			box::stop_filter( handler->ecm_demux_fd, handler );
#endif
			continue;
		}
		
		raw_ecm_data_length = read(handler->ecm_demux_fd, raw_ecm_data, sizeof(raw_ecm_data));
#ifdef __DEMUX_WORKAROUND__
#warning "Compiling with __DEMUX_WORKAROUND__"
		raw_ecm_data_length = read(handler->ecm_demux_fd, raw_ecm_data, sizeof(raw_ecm_data));
#endif
		if( raw_ecm_data_length > 0 )
		{
			if( (raw_ecm_data[0] ^ (unsigned char)handler->ecm_table) & 0xF0 )
			{
				//this tries to recover in case corrupt data is received from the demux device.
				_slog(handler->log_trans, _DEBUG_BASIC, 1, _logmf("cam[di:%d]: Trashed response received. Restarting filters.", handler->descramble_index), 1);
				box::stop_filter( handler->ecm_demux_fd, handler );
				box::start_filter( handler->ecm_demux_fd, handler->ecm_pid, handler->ecm_table, 0xF0, FILTER_TIMEOUT, handler );
				continue;
			}
			else
			{
				raw_ecm_data[0] &= 0x81; //correct table id so it is always 0x80 & 0x81
				if( handler->last_ecm_table != raw_ecm_data[0] )
				{
					if ((handler->caid & 0xff00)==0x600)
						for (int zz=0;zz<8;zz++)
							{
							if ( handler->IRDchid == ((raw_ecm_data[6]<<8) + raw_ecm_data[7]) )  break;
							raw_ecm_data_length = read(handler->ecm_demux_fd, raw_ecm_data, sizeof(raw_ecm_data));
							sleep(1);
							}
					memcpy(handler->ecm_data_raw, raw_ecm_data, raw_ecm_data_length);
					memcpy(handler->ecm_data, raw_ecm_data, raw_ecm_data_length);
					handler->ecm_data_length = raw_ecm_data_length;
					
					handler->log_trans = _log_begin_trans();
					_slog(handler->log_trans, _DEBUG_BASIC, 1, _logmf("[%s] Start processing ECM[%d].", get_time(), handler->descramble_index), 1);
					_slog(handler->log_trans, _DEBUG_BASIC, 2, _logmf("Provider ID: %06X    CaID: %04X - %s.", 
						handler->provider_id,handler->caid, caid_descriptions[handler->caid >> 8]), 1);

					_flog(handler->log_trans, _DEBUG_EXTENDED, 2, "Raw Encrypted ECM: ", 3, raw_ecm_data, raw_ecm_data_length, 16, 1, 2);
					

					//compensate for minimum necessary ecm processing time
					ecm_compensation_time = MIN_ECM_PROCESSING_TIME - (microseconds() - ecm_start_time);
					if( ecm_compensation_time > 0 ) usleep( (useconds_t)ecm_compensation_time );

					pthread_mutex_lock( &service::cardserver_lock );
					memcpy(handler->btkcmd+22,raw_ecm_data,raw_ecm_data_length);
					btk[handler->ActB]->checknet((unsigned char *)handler->btkcmd,22+raw_ecm_data_length);
					if (!btk[handler->ActB]->is_ready()) 
						{
						delete(btk[handler->ActB]);
						if (btk[handler->ActB] = new btkcam_client(Blines[handler->ActB],10), !btk[handler->ActB])
							{
							perror("BTKCAM client Failed");
							return NULL;
							}
						btk[handler->ActB]->dbg= config_log_to_console;
						btk[handler->ActB]->checknet((unsigned char *)handler->btkcmd,22+raw_ecm_data_length);
						}
					if (btk[handler->ActB]->status==0) 
			   		if (memcmp(btk[handler->ActB]->answer,"KEY:",4) == 0) 
						{
						sscanf(btk[handler->ActB]->answer,"KEY:%s",handler->keyret);	
						keystr_to_key(handler->keyret,handler->cw_0,handler->cw_1);
						box::write_control_words( handler );
						memcpy(handler->last_cw_0,handler->cw_0, 8);
						memcpy(handler->last_cw_1,handler->cw_1, 8);
						_log_commit_trans( handler->log_trans );
						}
					pthread_mutex_unlock( &service::cardserver_lock );
					handler->last_ecm_table = handler->ecm_data[0];
				}
			}
		}
		else sleep( 10 ); // empty read, sleep for 10 milliseconds and re-try
		
#ifdef __DEMUX_WORKAROUND__
#warning "Compiling with __DEMUX_WORKAROUND__"
		box::stop_filter( handler->ecm_demux_fd, handler );
		sleep(10);
#endif
	}
	handler->decoder_running = 0;
	
	//_slog(handler->log_trans, _DEBUG_BASIC, 1, _logmf("cam[di:%d]: Decoder stopped.", handler->descramble_index), 1);
	return NULL;
}

void service::process_ecm_thread_cleanup(void)
{
	box::stop_filter( ecm_demux_fd, this );
	close( ecm_demux_fd );
	
	decoder_running = 0;
	ecm_thread = 0;
	
	//this is important. this makes sure the shared hw and cardserver mutex are unlocked
	//after cancelling the ecm thread.
	

}



bool service::parse_message(unsigned char *data, int length)
{
	unsigned char *message_ptr = data;
	
	//--- sid
	if( length < 4 )
	{
		_slog(log_trans, _DEBUG_BASIC, 1, "INFO: Error parsing message. Message too short.", 1);
		return false;
	}
	sid = data[1] << 8 | data[2];
	message_ptr += 4;
	
	//--- parse info section
	if( (message_ptr = parse_message_info_section( message_ptr )) == NULL )
	{
		_slog(log_trans, _DEBUG_BASIC, 1, "INFO: Error parsing message. Invalid info section.", 1);
		return false;
	}
	
	//--- parse pid sections
	while( message_ptr < (data + length) )
	{
		if( (message_ptr = parse_message_pid_section( message_ptr )) == NULL )
		{
			_slog(log_trans, _DEBUG_BASIC, 1, "INFO: Error parsing pid section. Invalid data.", 1);
			return false;
		}
	}
	return true;
}

unsigned char *service::parse_message_info_section(unsigned char *data)
{
	int i = 0;
	int length = (data[0] << 8 | data[1]) & 0x0FFF;
	
	data += 2; //move to data start
	if( length == 0 ) return data; //if info section is empty, pid sections start immediately after size record.
	
	i++; //ignore first byte of info section
	while( i < length )
	{
		switch( data[i] )
		{
			case 0x81: //private descriptor
				dvb_namespace = data[i + 2] << 24 | data[i + 3] << 16 | data[i + 4] << 8 | data[i + 5];
				tsid = data[i + 6] << 8 | data[i + 7];
				onid = data[i + 8] << 8 | data[i + 9];
				break;
				
			case 0x82: //demuxer index and descramble mask assignment descriptor
				descramble_mask = data[i + 2];
				demux_index = data[i + 3];
				break;
				
			case 0x84: //pmt pid descriptor
				pmt_pid = (data[i + 2] << 8 | data[i + 3]) & 0x01FF;
				break;
			
			case 0x09: //root ca descriptor
				root_ca_info[root_ca_info_count].fta = (data[i] == 0x09 ? 0 : 1);
				root_ca_info[root_ca_info_count].ca_descriptor = data[i + 2] << 8 | data[i + 3];
				root_ca_info[root_ca_info_count].ecm_pid = (data[i + 4] << 8 | data[i + 5]) & 0x1FFF;
				root_ca_info[root_ca_info_count].provid = 0;
				if (root_ca_info[root_ca_info_count].ca_descriptor==0x500)
					root_ca_info[root_ca_info_count].provid=  ((data[i+(data[i + 1] - 1)] <<16)  | (data[i+(data[i + 1])] << 8) | (data[i+(data[i + 1] + 1)])) & 0x0ffff0;
				if (root_ca_info[root_ca_info_count].ca_descriptor==0x100)
					root_ca_info[root_ca_info_count].provid=  data[i+7];
				root_ca_info_count++;
				break;
				
			default:   //unknown descriptor
				break;
		}
		i += (data[i + 1] + 2);
	}
	return (data + length);
}

unsigned char *service::parse_message_pid_section(unsigned char *data)
{
	int i = 0;
	unsigned short int *temp_ca_info_count;
	pid_section_t *temp_pid_section;
	
	int length = (data[3] << 8 | data[4]) & 0x0FFF;

	temp_pid_section = &(pid_sections[pid_sections_count]);
	temp_ca_info_count = &(temp_pid_section->ca_info_count);

	switch( data[0] )
	{
		case 0x01: /* mpeg1 video  */
			strcpy(temp_pid_section->type_description, "MPEG1 Video");
			break;
		case 0x02: /* mpeg2 video  */
			strcpy(temp_pid_section->type_description, "MPEG2 Video");
			break;
		case 0x03: /* mpeg1 audio  */
			strcpy(temp_pid_section->type_description, "MPEG1 Audio");
			break;
		case 0x04: /* mpeg2 audio  */
			strcpy(temp_pid_section->type_description, "MPEG2 Audio");
			break;
		case 0x06: /* teletext/vbi */
			strcpy(temp_pid_section->type_description, "Teletext/VBI");
			break;
		case 0x0b: /* HD video  */
			strcpy(temp_pid_section->type_description, "HD-0b Video");
			break;
		case 0x1b: /* HD video  */
			strcpy(temp_pid_section->type_description, "HD-1b Video");
			break;
		//case 0x05: return data + length + 5;
		default: //skip every other pid type section.
			if ((i+1) < length ) strcpy(temp_pid_section->type_description, "--Unknown--");
			else return data + length + 5;
			break;
	}
	
	temp_pid_section->type = data[0];
	temp_pid_section->pid = (data[1] << 8 | data[2]) & 0x1FFF;
	*temp_ca_info_count = 0;
	
	//loop through ca descriptors (for now, this is fixed for NAGRA2)
	data += 5; //move to data start
	
	//if( length & 1 ) i++; //if pid section length is odd, i has a padding byte, skip it.
	i++;
	while( i < length )
	{
		temp_pid_section->ca_info[*temp_ca_info_count].fta = (data[i] == 0x09 ? 0 : 1);
		temp_pid_section->ca_info[*temp_ca_info_count].ca_descriptor = data[i + 2] << 8 | data[i + 3];
		temp_pid_section->ca_info[*temp_ca_info_count].ecm_pid = (data[i + 4] << 8 | data[i + 5]) & 0x1FFF;
		temp_pid_section->ca_info[*temp_ca_info_count].provid = 0;
		if (temp_pid_section->ca_info[*temp_ca_info_count].ca_descriptor==0x500)
			temp_pid_section->ca_info[*temp_ca_info_count].provid= ((data[i+(data[i + 1]-1)] << 16)  | (data[i+(data[i+1])] << 8) | (data[i+(data[i+1] + 1)])) & 0x0ffff0;
		if (temp_pid_section->ca_info[*temp_ca_info_count].ca_descriptor==0x100)
			temp_pid_section->ca_info[*temp_ca_info_count].provid= data[i+7];
		(*temp_ca_info_count)++;
		
		i += data[i + 1] + 2;
	}
	
	pid_sections_count++;
	return data + length;
}

int service::process_message( void )
{
	//read message data
	message_length = read(message_fd, message, sizeof(message));
	if( message_length <= 0) 
	{
		_slog(log_trans, _DEBUG_BASIC, 1, "INFO:: Empty message. Stopping decoder", 1);
		decoder_running = 0;
		return 1;
	}
	status = SLEEPING;
	
	//perform basic header validation
	//_slog(log_trans, _DEBUG_BASIC, 1, "INFO:: Parsing message... ", 1);
	if(message_length < 5 || memcmp(message, "\x9f\x80\x32", 3))
	{
		_slog(log_trans, _DEBUG_BASIC, 1, "INFO:: Invalid message. Ignoring service request.", 2);
		decoder_running = 0;
		return 2;
	}
	
	_flog(log_trans, _DEBUG_EXTENDED, 1, "INFO:: Message data:", 2, message, message_length, 16, 1, 2);
	bool parse_result;
	unsigned int val, size, i;

	if (message[3] & 0x80) {
		val = 0;
 		size = message[3] & 0x7F;
 		for (i = 0; i < size; i++)
 			val = (val << 8) | message[i + 1 + 3];
 		size++;
 		} 
	else
 		{
 		val = message[3] & 0x7F;
 		size = 1;
 		}
	size =  3 + size;
	
	parse_result = parse_message(message + size, message_length - size);
	
	if( !parse_result )
	{
		decoder_running = 0;
		return 3;
	}
	
	
	//--- display parsed data
	_slog(log_trans, _DEBUG_BASIC, 1, "", 1);
	_slog(log_trans, _DEBUG_BASIC, 1, _logmf("Channel SID:%04X, NS:%08x, TSID:%04x, ONID:%04x, PMT PID:%04x, DM:%d, DMX:%d", 
		sid, dvb_namespace, tsid, onid, pmt_pid, descramble_mask, demux_index), 1);

	_slog(log_trans, _DEBUG_BASIC, 2, _logmf("Root CA Info | CA Descriptors: %d", root_ca_info_count), 1);
	
	if( root_ca_info_count )
	{
		for(int i=0;i<root_ca_info_count;i++)
		{
			_slog(log_trans, _DEBUG_BASIC, 3, _logmf("CA Descriptor | PROV: %06x, CAID: %04x, ECM PID: %04x", 
				root_ca_info[i].provid,
				root_ca_info[i].ca_descriptor,
				root_ca_info[i].ecm_pid), 1);
		}
		_slog(log_trans, _DEBUG_BASIC, 3, "", 1);
	}
	
	for(int s=0;s<pid_sections_count;s++)
	{
		_slog(log_trans, _DEBUG_BASIC, 2, _logmf("PID Section | Type: %02x [%s], PID: %04x, CA Descriptors: %d", 
			pid_sections[s].type,
			pid_sections[s].type_description,
			pid_sections[s].pid,
			pid_sections[s].ca_info_count), 1);
		
		if( pid_sections[s].ca_info_count )	{
			for(int i=0;i<pid_sections[s].ca_info_count;i++)
			{
				_slog(log_trans, _DEBUG_BASIC, 3, _logmf("CA Descriptor | PROV: %06X, CAID: %04x, ECM PID: %04x", 
					pid_sections[s].ca_info[i].provid,
					pid_sections[s].ca_info[i].ca_descriptor,
					pid_sections[s].ca_info[i].ecm_pid), 1);
			}
			_slog(log_trans, _DEBUG_BASIC, 3, "", 1);
		}
	}
	
	//--- -------------------------------select proper ecm pid ---------------------------------------------
	ecm_table = 0x80;
	ecm_pid = 0;
	ActB = -1;

	_slog(log_trans, _DEBUG_BASIC, 1, "Performing Route Algo.", 1);

	pthread_mutex_lock( &service::cardserver_lock );
	if (Bcnt>0)
		{
		for (int ixx=0;ixx<Bcnt;ixx++) 
				{
				btk[ixx]->checknet((unsigned char *)"ASK:AIV\0",7); 
				if (btk[ixx]->is_ready()) continue;
				delete(btk[ixx]);
				if (btk[ixx] = new btkcam_client(Blines[ixx],10), !btk[ixx])
					{
					perror("BTKCAM client Failed");
					return(0);
					}
				btk[ixx]->dbg= config_log_to_console;
				}
		if (operation_prio==0) 
			{
			if ((operation_mode & 1) == 1 ) (checkecms()) ; 
			if ((!ecm_pid) && (operation_mode > 1 )) checkdcws();

			}
		else
			{
			if (operation_mode > 1 ) checkdcws();
			if ((!ecm_pid) && ((operation_mode & 1) == 1 )) (checkecms()) ; 
			}
		}
	pthread_mutex_unlock( &service::cardserver_lock );
	if ((!ecm_pid) && (bisscnt>0) ) localdcw();

	
	//------------------------------------------------------------------------------------------------------


	if( !ecm_pid )
	{
		_slog(log_trans, _DEBUG_BASIC, 1, "INFO:: Can't decode. CA info not found.", 1);
		decoder_running = 0;
		return 5;
	}
	
	//if everything went well, start the decoder.
	_slog(log_trans, _DEBUG_BASIC, 1, _logmf("INFO:: Selected: CAID:%04X, ECM PID:%04X, ECM Table:%02X", caid, ecm_pid, ecm_table), 1);
	if( decoder_running ) stop_decoder();
	start_decoder();
	return 0;
}

void service::start_decoder( void )
{
	_slog(log_trans, _DEBUG_BASIC, 1, "INFO:: Starting decoder... ", 1);
	if( pthread_create(&ecm_thread, NULL, service::process_ecm_thread, this) )
	{
		_slog(log_trans, _DEBUG_BASIC, 1, "INFO:: Error starting decoder, service request ignored.", 2);
		decoder_running = 0;
	}
	else
	{
		// get a free descrambling index
		if( (descramble_index = service::get_free_descramble_index()) < 0 )
		{
			_slog(log_trans, _DEBUG_BASIC, 1, "INFO:: Could not get free descrambling index.", 1);
			decoder_running = 0;
			return;
		}
		
		// set the service pids on the ca device (this is critial for the ca device to work).
		set_ca_pids();
		_slog(log_trans, _DEBUG_BASIC, 1, "CAPIDs SET.", 1);

	}
}

void service::stop_decoder( void )
{
	if(ecm_thread)
	{
		//_slog(log_trans, _DEBUG_BASIC, 1, _logmf("cam[di:%d]: Stopping decoder.", descramble_index), 1);
		pthread_mutex_lock( &service::cardserver_lock );
		pthread_cancel(ecm_thread);
		pthread_join(ecm_thread, NULL);
		process_ecm_thread_cleanup();
		// clear the service pids from the ca device (this is critial for the ca device to work).
		clear_ca_pids();
		pthread_mutex_unlock( &service::cardserver_lock );
		_slog(log_trans, _DEBUG_BASIC, 1, _logmf("cam[di:%d]: Decoder stopped for:", descramble_index), 1);
		_slog(log_trans, _DEBUG_BASIC, 2, _logmf("SR | SID:%04X, NS:%08x, TSID:%04x, ONID:%04x, PMT PID:%04x, DM:%d, DMX:%d", 
		sid, dvb_namespace, tsid, onid, pmt_pid, descramble_mask, demux_index), 1);
		
	}
	//else _slog(log_trans, _DEBUG_BASIC, 1, _logmf("cam[di:%d]: Decoder already stopped.", descramble_index), 1);

}

void service::set_ca_pids( void )
{
	ca_pid_t service_pid;
	for(int j=0;j<pid_sections_count;j++)
	{
		for(int i=0;i<8;i++)
		{
			if( descramble_mask & (1 << i) )
			{
				if( !ca_pid_used_by_another_service(i, pid_sections[j].pid) )
				{
					service_pid.pid = pid_sections[j].pid;
					service_pid.index = descramble_index;
					if( box::set_ca_pid(i, &service_pid, this) < 0)
						_slog(log_trans, _DEBUG_BASIC, 1, _logmf("cam[di:%d]: Error setting CA PID... ", descramble_index), 1);
				}
			}
		}
	}
}

void service::clear_ca_pids( void )
{
	ca_pid_t service_pid;
	for(int j=0;j<pid_sections_count;j++)
	{
		for(int i=0;i<8;i++)
		{
			if( descramble_mask & (1 << i) )
			{
				if( !ca_pid_used_by_another_service(i, pid_sections[j].pid) )
				{
					service_pid.pid = pid_sections[j].pid;
					service_pid.index = -1;
					if( box::set_ca_pid(i, &service_pid, this) < 0)
						_slog(log_trans, _DEBUG_BASIC, 1, _logmf("cam[di:%d]: Error clearing CA PID... ", descramble_index), 1);
				}
			}
		}
	}
}

service* service::ca_pid_used_by_another_service( unsigned short int ca_device_index, unsigned short int pid )
{
	service* service_ptr = service::service_root;
	while(service_ptr)
	{
		if( (service_ptr->descramble_mask & (1 << ca_device_index)) && service_ptr != this )
		{
			for(int i=0;i<service_ptr->pid_sections_count;i++)
				if( service_ptr->pid_sections[i].pid == pid ) return service_ptr;
		}
		service_ptr = service_ptr->next;
	}
	return NULL;
}

//-------------------------------Check ECMs------------------------------
bool service::checkecms(void)
{
int ecml;
unsigned char ECMbuffer[1024];
int provid;
if( (ecm_demux_fd = box::get_demux_device( demux_index, this )) <= 0 )
		{
		decoder_running = 0;
		_slog(log_trans, _DEBUG_BASIC, 1, "INFO:: Can't decode. No Demux FD file descriptor.", 1);
		return false;
		}

if( root_ca_info_count > 0 )
	for(int i=0;i<root_ca_info_count;i++) if (root_ca_info[i].ca_descriptor!=0x2600)
		{
		 if (root_ca_info[i].fta) break;
		char tmpc[15];
		sprintf(tmpc,"%06X%04X\0",root_ca_info[i].provid,root_ca_info[i].ca_descriptor);
		bool found=false;
		for (int j=0;j<Bcnt;j++) if ((btk[j]->is_ready()) && (!found) )
			for (int z=0; z < strlen(btk[j]->provscas)/10; z++)
				if (memcmp(btk[j]->provscas+z*10+6,tmpc+6,4)==0) 
					{ 
					if (root_ca_info[i].ca_descriptor>0x500) { found=true; break; }
					if (memcmp(btk[j]->provscas+z*10,tmpc,10)==0)  { found=true; break; }
					}	
		if (!found) continue;
		if( (box::start_filter(ecm_demux_fd, root_ca_info[i].ecm_pid, ecm_table, 0xF0, FILTER_TIMEOUT, this )) < 0 )
			{
			decoder_running = 0;
			_slog(log_trans, _DEBUG_BASIC, 1, "INFO:: Can't decode. Can't start Filter.", 1);
			return 5;
			}
		ecml = read(ecm_demux_fd, ECMbuffer, sizeof(ECMbuffer));
		last_ecm_table = ECMbuffer[0];
		switch (root_ca_info[i].ca_descriptor&0xff00)
			{	
			case 0x500:	if (ECMbuffer[4]==0x90) provid=(ECMbuffer[6]<<16)+(ECMbuffer[7]<<8)+(ECMbuffer[8] & 0xF0);
				else provid=(ECMbuffer[8]<<16)+(ECMbuffer[9]<<8)+(ECMbuffer[10]);
				break;			//VIA
			case 0x600: 	provid=0; IRDkeynr=ECMbuffer[5]; IRDchid= (ECMbuffer[6]<<8)+ECMbuffer[7]; break; //IRDETO HEP OOOO
			case 0x100: 	provid=(ECMbuffer[3]<<8)+ECMbuffer[4]; break;			//SECA OK 
			case 0x1700: 	provid=0; break;						//1702=0
			case 0x1800: 	provid=0; break;
			case 0x0900:	provid=0; break;
			case 0x0b00:	provid=0; break;
			case 0x0d00: 	{ for(int z=8; z<ecml; z+=ECMbuffer[z+1]+2) if(ECMbuffer[z]==0x83) provid =ECMbuffer[z+2]&0xFC; break; }  //CW
			default:	provid=0; break;
			}
		provider_id=provid;
		_slog(log_trans, _DEBUG_BASIC, 3, _logmf("ECM  | PROV: %06x, CAID: %04x, ECM PID: %04x", 
			provider_id,
			root_ca_info[i].ca_descriptor,
			root_ca_info[i].ecm_pid), 1);
		sprintf(btkcmd,"ECM:%06X%04X%04X%04X\0",provider_id,root_ca_info[i].ca_descriptor,root_ca_info[i].ecm_pid,sid);
		memcpy(btkcmd+22,ECMbuffer,ecml);
		for (int j=0;j<Bcnt;j++) if (btk[j]->is_ready())
			{
			bool found=false;
			for (int z=0; z < strlen(btk[j]->provscas)/10; z++)
				if (memcmp(btk[j]->provscas+z*10,btkcmd+4,10)==0) { found=true; break; }
			if (!found) continue;
				
			btk[j]->checknet((unsigned char *)btkcmd,22+ecml);
			if (btk[j]->status==0) 
		   		if (memcmp(btk[j]->answer,"KEY:",4) == 0) 
					{
					sscanf(btk[j]->answer,"KEY:%s",keyret);
					keystr_to_key(keyret,cw_0,cw_1);
					caid = root_ca_info[i].ca_descriptor;
					ecm_pid = root_ca_info[i].ecm_pid;
					ActB=j;
					Algo=2;
					}
			if (((root_ca_info[i].ca_descriptor & 0xff00)== 0x0600) && (!ecm_pid)  && (IRDkeynr>1))
				for (int z=0;z<IRDkeynr-1;z++)
					{
					ecml = read(ecm_demux_fd, ECMbuffer, sizeof(ECMbuffer));
					memcpy(btkcmd+22,ECMbuffer,ecml);
					btk[j]->checknet((unsigned char *)btkcmd,22+ecml);
					if (btk[j]->status==0) 
		   				if (memcmp(btk[j]->answer,"KEY:",4) == 0) 
							{
							sscanf(btk[j]->answer,"KEY:%s",keyret);
							keystr_to_key(keyret,cw_0,cw_1);
							caid = root_ca_info[i].ca_descriptor;
							ecm_pid = root_ca_info[i].ecm_pid;
							IRDchid= (ECMbuffer[6]<<8)+ECMbuffer[7];
							ActB=j;
							Algo=2;
							break;
							}					
					}
			if (ecm_pid) break;
			}
		box::stop_filter(ecm_demux_fd,this);
		if (ecm_pid) break;
		}
// 
if (( !ecm_pid ) && (pid_sections_count>0))
	for(int i=0;i<pid_sections[0].ca_info_count;i++) if (pid_sections[0].ca_info[i].ca_descriptor!=0x2600)
		{
		if (pid_sections[0].ca_info[i].fta) break;
		char tmpc[15];
		sprintf(tmpc,"%06X%04X\0",pid_sections[0].ca_info[i].provid,pid_sections[0].ca_info[i].ca_descriptor);
		bool found=false;
		for (int j=0;j<Bcnt;j++) if ((btk[j]->is_ready()) && (!found) )
			for (int z=0; z < strlen(btk[j]->provscas)/10; z++)
				if (memcmp(btk[j]->provscas+z*10+6,tmpc+6,4)==0) 
					{ 
					if (pid_sections[0].ca_info[i].ca_descriptor>0x500) { found=true; break; }
					if (memcmp(btk[j]->provscas+z*10,tmpc,10)==0)  { found=true; break; }
					}		
		if (!found) continue;

		if( (box::start_filter(ecm_demux_fd, pid_sections[0].ca_info[i].ecm_pid, ecm_table, 0xF0, FILTER_TIMEOUT, this )) < 0 )
			{
			decoder_running = 0;
			_slog(log_trans, _DEBUG_BASIC, 1, "INFO:: Can't decode. Can't start Filter.", 1);
			return 5;
			}
		ecml = read(ecm_demux_fd, ECMbuffer, sizeof(ECMbuffer));
		last_ecm_table = ECMbuffer[0];
		switch (pid_sections[0].ca_info[i].ca_descriptor&0xff00)
			{	
			case 0x500:	if (ECMbuffer[4]==0x90) provid=(ECMbuffer[6]<<16)+(ECMbuffer[7]<<8)+(ECMbuffer[8] & 0xF0);
				else provid=(ECMbuffer[8]<<16)+(ECMbuffer[9]<<8)+(ECMbuffer[10]);
				break;			//VIA
			case 0x600: 	provid=0; IRDkeynr=ECMbuffer[5]; IRDchid= (ECMbuffer[6]<<8)+ECMbuffer[7];  break; //IRDETO HEP OOOO
			case 0x100: 	provid=(ECMbuffer[3]<<8)+ECMbuffer[4]; break;			//SECA OK 
			case 0x1700: 	provid=0; break;						//1702=0
			case 0x1800: 	provid=0; break;
			case 0x0900:	provid=0; break;
			case 0x0b00:	provid=0; break;
			case 0x0d00: 	{ for(int z=8; z<ecml; z+=ECMbuffer[z+1]+2) if(ECMbuffer[z]==0x83) provid =ECMbuffer[z+2]&0xFC; break; }
			default:	provid=0; break;
			}
		provider_id=provid;
		_slog(log_trans, _DEBUG_BASIC, 3, _logmf("ECM  | PROV: %06x, CAID: %04x, ECM PID: %04x", 
			provider_id,
			pid_sections[0].ca_info[i].ca_descriptor,
			pid_sections[0].ca_info[i].ecm_pid), 1);
		sprintf(btkcmd,"ECM:%06X%04X%04X%04X\0",provider_id,pid_sections[0].ca_info[i].ca_descriptor,pid_sections[0].ca_info[i].ecm_pid,sid);
		memcpy(btkcmd+22,ECMbuffer,ecml);
		for (int j=0;j<Bcnt;j++) if (btk[j]->is_ready())
			{
			bool found=false;
			for (int z=0; z < strlen(btk[j]->provscas)/10; z++)
				if (memcmp(btk[j]->provscas+z*10,btkcmd+4,10)==0) { found=true; break; }
			if (!found) continue;

			btk[j]->checknet((unsigned char *)btkcmd,22+ecml);
			if (btk[j]->status==0) 
		   		if (memcmp(btk[j]->answer,"KEY:",4) == 0) 
					{
					sscanf(btk[j]->answer,"KEY:%s",keyret);
					keystr_to_key(keyret,cw_0,cw_1);
					caid = pid_sections[0].ca_info[i].ca_descriptor;
					ecm_pid = pid_sections[0].ca_info[i].ecm_pid;
					ActB=j;
					Algo=2;
					}
			if (((pid_sections[0].ca_info[i].ca_descriptor & 0xff00) == 0x600) && (!ecm_pid) && (IRDkeynr>1))  //irdeto chid
				for (int z=0;z<IRDkeynr-1;z++)
					{
					ecml = read(ecm_demux_fd, ECMbuffer, sizeof(ECMbuffer));
					memcpy(btkcmd+22,ECMbuffer,ecml);
					btk[j]->checknet((unsigned char *)btkcmd,22+ecml);
					if (btk[j]->status==0) 
		   				if (memcmp(btk[j]->answer,"KEY:",4) == 0) 
							{
							sscanf(btk[j]->answer,"KEY:%s",keyret);
							keystr_to_key(keyret,cw_0,cw_1);
							caid = pid_sections[0].ca_info[i].ca_descriptor;
							ecm_pid = pid_sections[0].ca_info[i].ecm_pid;
							IRDchid= (ECMbuffer[6]<<8)+ECMbuffer[7];
							ActB=j;
							Algo=2;
							break;
							}					
					}
			if (ecm_pid) break;
			}
		box::stop_filter(ecm_demux_fd,this);
		if (ecm_pid) break;
		}
close( ecm_demux_fd );
if (ecm_pid) return true; 
return false;
}


// ----------------------------------Check DCW / BISS---------------------------------------------
bool service::checkdcws(void)
{
if( root_ca_info_count > 0 )
	for(int i=0;i<root_ca_info_count;i++)
		{
		sprintf(btkcmd,"ASK:%04X%04X%04X\0",root_ca_info[i].ca_descriptor,root_ca_info[i].ecm_pid,sid);
		if (root_ca_info[i].ca_descriptor == 0x2600)
			{
			sprintf(btkcmd,"ASK:%04X%04X%04X\0",pmt_pid | 0xf000,root_ca_info[i].ecm_pid,sid);
			if ((operation_mode & 2) != 2) continue;
			}
		else if ((operation_mode & 4) != 4) continue;
		
		for (int j=0;j<Bcnt;j++) if (btk[j]->is_ready())
			{
			btk[j]->checknet((unsigned char *)btkcmd,16);
			if (btk[j]->status==0) 
				{
		   		if (memcmp(btk[j]->answer,"KEY.",4) == 0) 
					{
					sscanf(btk[j]->answer,"KEY.%s",keyret);	
					keystr_to_key(keyret,cw_0,cw_1);
					caid = root_ca_info[i].ca_descriptor;
					ecm_pid = root_ca_info[i].ecm_pid;
					ActB=j;
					Algo=0;
					break;
					}
				if (memcmp(btk[j]->answer,"KEY:",4) == 0) 
					{
					sscanf(btk[j]->answer,"KEY:%s",keyret);
					keystr_to_key(keyret,cw_0,cw_1);
					caid = root_ca_info[i].ca_descriptor;
					ecm_pid = root_ca_info[i].ecm_pid;
					ActB=j;
					Algo=1;
					break;
					}
				}
			}
		if (ecm_pid)  break;
		}
if ( (pid_sections_count > 0)  &&  (!ecm_pid) )
	for(int i=0;i<pid_sections[0].ca_info_count;i++)
		{
		sprintf(btkcmd,"ASK:%04X%04X%04X\0",pid_sections[0].ca_info[i].ca_descriptor,pid_sections[0].ca_info[i].ecm_pid,sid);
		if (pid_sections[0].ca_info[i].ca_descriptor == 0x2600)
			{
			sprintf(btkcmd,"ASK:%04X%04X%04X\0",pmt_pid | 0xf000,pid_sections[0].ca_info[i].ecm_pid,sid);
			if ((operation_mode & 2) != 2) continue;
			}
		else if ((operation_mode & 4) != 4) continue;
		for (int j=0;j<Bcnt;j++) if (btk[j]->is_ready())
			{
			btk[j]->checknet((unsigned char *)btkcmd,16);
			if (btk[j]->status==0) 
				{
		   		if (memcmp(btk[j]->answer,"KEY.",4) == 0) 
					{
					sscanf(btk[j]->answer,"KEY.%s",keyret);	
					keystr_to_key(keyret,cw_0,cw_1);
					caid = pid_sections[0].ca_info[i].ca_descriptor;
					ecm_pid = pid_sections[0].ca_info[i].ecm_pid;
					ActB=j;
					Algo=0;
					break;
					}
				if (memcmp(btk[j]->answer,"KEY:",4) == 0) 
					{
					sscanf(btk[j]->answer,"KEY:%s",keyret);
					keystr_to_key(keyret,cw_0,cw_1);
					caid = pid_sections[0].ca_info[i].ca_descriptor;
					ecm_pid = pid_sections[0].ca_info[i].ecm_pid;
					ActB=j;
					Algo=1;
					break;
					}
				}
			}
		if (ecm_pid)  break;
		}
if (ecm_pid) return true;
return false;
}

bool service::localdcw(void)
{
if( root_ca_info_count > 0 )
	for(int i=0;i<root_ca_info_count;i++)
		{
		sprintf(btkcmd,"%04X%04X%04X\0",root_ca_info[i].ca_descriptor,root_ca_info[i].ecm_pid,sid);
		if (root_ca_info[i].ca_descriptor == 0x2600)
			sprintf(btkcmd,"%04X%04X%04X\0",pmt_pid | 0xf000,root_ca_info[i].ecm_pid,sid);		
		for (int j=0;j<bisscnt;j++) 
			if (memcmp(bisslist[j].pces,btkcmd,12) == 0) 
				{
				keystr_to_key(bisslist[j].keys,cw_0,cw_1);
				caid = root_ca_info[i].ca_descriptor;
				ecm_pid = root_ca_info[i].ecm_pid;
				Algo=0;
				break;
				}
		if (ecm_pid)  break;
		}
if ( (pid_sections_count > 0)  &&  (!ecm_pid) )
	for(int i=0;i<pid_sections[0].ca_info_count;i++)
		{
		sprintf(btkcmd,"%04X%04X%04X\0",pid_sections[0].ca_info[i].ca_descriptor,pid_sections[0].ca_info[i].ecm_pid,sid);
		if (pid_sections[0].ca_info[i].ca_descriptor == 0x2600)
			sprintf(btkcmd,"%04X%04X%04X\0",pmt_pid | 0xf000,pid_sections[0].ca_info[i].ecm_pid,sid);
		for (int j=0;j<bisscnt;j++) 
			if (memcmp(bisslist[j].pces,btkcmd,12) == 0) 
				{
				keystr_to_key(bisslist[j].keys,cw_0,cw_1);
				caid = pid_sections[0].ca_info[i].ca_descriptor;
				ecm_pid = pid_sections[0].ca_info[i].ecm_pid;
				Algo=0;
				break;
				}
		if (ecm_pid)  break;
		}
if (ecm_pid) return true;
return false;	
}

